@@ -1,5 +1,18 @@
-/*
- * Copyright 2001 Computing Research Labs, New Mexico State University
+/* $OpenLDAP: pkg/ldap/libraries/liblunicode/ucdata/ucpgba.c,v 1.3.4.2 2004/01/01 18:16:31 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1998-2004 The OpenLDAP Foundation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
+ */
+/* Copyright 2001 Computing Research Labs, New Mexico State University
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -19,18 +32,13 @@
  * OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
  * THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  */
-#ifndef lint
-#ifdef __GNUC__
-static char rcsid[] __attribute__ ((unused)) = "$Id: ucpgba.c,v 1.3 2003/07/18 01:10:28 jtownsen Exp $";
-#else
-static char rcsid[] = "$Id: ucpgba.c,v 1.3 2003/07/18 01:10:28 jtownsen Exp $";
-#endif
-#endif
+/* $Id: ucpgba.c,v 1.6 2005/07/21 03:29:10 jtownsen Exp $ */
 
 #include "portable.h"
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <ac/bytes.h>
 
 #include "ucdata.h"
 #include "ucpgba.h"
